@@ -46,30 +46,32 @@ MultiFormatReader::MultiFormatReader(const ReaderOptions& opts) : _opts(opts)
 
 MultiFormatReader::~MultiFormatReader() = default;
 
-Result
-MultiFormatReader::read(const BinaryBitmap& image) const
+Barcode MultiFormatReader::read(const BinaryBitmap& image) const
 {
-	Result r;
+	Barcode r;
 	for (const auto& reader : _readers) {
 		r = reader->decode(image);
   		if (r.isValid())
 			return r;
 	}
-	return _opts.returnErrors() ? r : Result();
+	return _opts.returnErrors() ? r : Barcode();
 }
 
-Results MultiFormatReader::readMultiple(const BinaryBitmap& image, int maxSymbols) const
+Barcodes MultiFormatReader::readMultiple(const BinaryBitmap& image, int maxSymbols) const
 {
-	std::vector<Result> res;
+	Barcodes res;
 
 	for (const auto& reader : _readers) {
 		if (image.inverted() && !reader->supportsInversion)
 			continue;
 		auto r = reader->decode(image, maxSymbols);
 		if (!_opts.returnErrors()) {
-			//TODO: C++20 res.erase_if()
-			auto it = std::remove_if(res.begin(), res.end(), [](auto&& r) { return !r.isValid(); });
-			res.erase(it, res.end());
+#ifdef __cpp_lib_erase_if
+			std::erase_if(r, [](auto&& s) { return !s.isValid(); });
+#else
+			auto it = std::remove_if(r.begin(), r.end(), [](auto&& s) { return !s.isValid(); });
+			r.erase(it, r.end());
+#endif
 		}
 		maxSymbols -= Size(r);
 		res.insert(res.end(), std::move_iterator(r.begin()), std::move_iterator(r.end()));
@@ -77,8 +79,8 @@ Results MultiFormatReader::readMultiple(const BinaryBitmap& image, int maxSymbol
 			break;
 	}
 
-	// sort results based on their position on the image
-	std::sort(res.begin(), res.end(), [](const Result& l, const Result& r) {
+	// sort barcodes based on their position on the image
+	std::sort(res.begin(), res.end(), [](const Barcode& l, const Barcode& r) {
 		auto lp = l.position().topLeft();
 		auto rp = r.position().topLeft();
 		return lp.y < rp.y || (lp.y == rp.y && lp.x < rp.x);
