@@ -5,16 +5,22 @@
 
 #pragma once
 
+#include "CharacterSet.h"
+
 #include <string>
 
 namespace ZXing {
 
-enum class CharacterSet;
-
 class TextEncoder
 {
+	static void GetBytes(const std::string& str, CharacterSet charset, std::string& bytes);
 	static void GetBytes(const std::wstring& str, CharacterSet charset, std::string& bytes);
 public:
+	static std::string FromUnicode(const std::string& str, CharacterSet charset) {
+		std::string r;
+		GetBytes(str, charset, r);
+		return r;
+	}
 	static std::string FromUnicode(const std::wstring& str, CharacterSet charset) {
 		std::string r;
 		GetBytes(str, charset, r);
