@@ -7,7 +7,8 @@
 #include <stddef.h>
 
 #include "BitArray.h"
-#include "oned/rss/ODRSSExpandedBinaryDecoder.h"
+#include "Error.h"
+#include "oned/ODDataBarExpandedBitDecoder.h"
 
 using namespace ZXing;
 
@@ -20,7 +21,11 @@ extern "C" int LLVMFuzzerTestOneInput(const uint8_t* data, size_t size)
 	for (size_t i = 0; i < size; ++i)
 		bits.appendBits(data[i], 8);
 
-	OneD::DataBar::DecodeExpandedBits(bits);
+	try {
+		OneD::DataBar::DecodeExpandedBits(bits);
+	} catch (std::out_of_range) {
+	} catch (Error) {
+	}
 
 	return 0;
 }
