@@ -6,7 +6,7 @@
 #include "oned/ODCode39Reader.h"
 
 #include "ReaderOptions.h"
-#include "Result.h"
+#include "Barcode.h"
 
 #include "gtest/gtest.h"
 
@@ -14,7 +14,7 @@ using namespace ZXing;
 using namespace ZXing::OneD;
 
 // Helper to call decodePattern()
-static Result parse(PatternRow row, ReaderOptions opts = {})
+static Barcode parse(PatternRow row, ReaderOptions opts = {})
 {
 	Code39Reader reader(opts);
 
@@ -38,34 +38,22 @@ TEST(ODCode39ReaderTest, SymbologyIdentifier)
 	{
 		// "A" with checksum
 		PatternRow row({ 2, 1, 1, 1, 1, 2, 1, 1, 2, 0, 2, 1, 1, 1, 1, 2, 1, 1, 2 });
-		auto result = parse(row, ReaderOptions().setValidateCode39CheckSum(true));
-		EXPECT_EQ(result.symbologyIdentifier(), "]A3");
-		EXPECT_EQ(result.text(), "A");
-
-		result = parse(row);
-		EXPECT_EQ(result.symbologyIdentifier(), "]A0");
+		auto result = parse(row);
+		EXPECT_EQ(result.symbologyIdentifier(), "]A1");
 		EXPECT_EQ(result.text(), "AA");
 	}
 	{
 		// Extended "a"
 		PatternRow row({ 1, 2, 1, 1, 1, 2, 1, 2, 1, 0, 2, 1, 1, 1, 1, 2, 1, 1, 2 });
-		auto result = parse(row, ReaderOptions().setTryCode39ExtendedMode(true));
+		auto result = parse(row);
 		EXPECT_EQ(result.symbologyIdentifier(), "]A4");
 		EXPECT_EQ(result.text(), "a");
-
-		result = parse(row);
-		EXPECT_EQ(result.symbologyIdentifier(), "]A0");
-		EXPECT_EQ(result.text(), "+A");
 	}
 	{
 		// Extended "a" with checksum
 		PatternRow row({ 1, 2, 1, 1, 1, 2, 1, 2, 1, 0, 2, 1, 1, 1, 1, 2, 1, 1, 2, 0, 2, 1, 1, 2, 1, 1, 2, 1, 1 });
-		auto result = parse(row, ReaderOptions().setTryCode39ExtendedMode(true).setValidateCode39CheckSum(true));
-		EXPECT_EQ(result.symbologyIdentifier(), "]A7");
-		EXPECT_EQ(result.text(), "a");
-
-		result = parse(row);
-		EXPECT_EQ(result.symbologyIdentifier(), "]A0");
-		EXPECT_EQ(result.text(), "+A8");
+		auto result = parse(row);
+		EXPECT_EQ(result.symbologyIdentifier(), "]A5");
+		EXPECT_EQ(result.text(), "a8");
 	}
 }
